@@ -91,11 +91,11 @@ static void		gst_colorspace_set_property		(GObject *object, guint prop_id,
 static void		gst_colorspace_get_property		(GObject *object, guint prop_id, 
 								 GValue *value, GParamSpec *pspec);
 
-static GstPadConnectReturn
+static GstPadLinkReturn
 			gst_colorspace_sinkconnect		(GstPad *pad, GstCaps *caps);
-static GstPadConnectReturn
+static GstPadLinkReturn
 			gst_colorspace_srcconnect 		(GstPad *pad, GstCaps *caps);
-static GstPadConnectReturn
+static GstPadLinkReturn
 			gst_colorspace_srcconnect_func 		(GstPad *pad, GstCaps *caps, gboolean newcaps);
 static void		gst_colorspace_chain			(GstPad *pad, GstBuffer *buf);
 static GstElementStateReturn
@@ -271,14 +271,19 @@ gst_colorspace_getcaps (GstPad *pad, GstCaps *caps)
   GstColorspace *space;
   GstCaps *result;
   GstCaps *peercaps;
-  GstCaps *ourcaps;
+  GstCaps *ourcaps, *temp;
   
   space = GST_COLORSPACE (gst_pad_get_parent (pad));
 
   /* we can do everything our peer can... */
-  peercaps = gst_caps_copy (gst_pad_get_allowed_caps (space->srcpad));
+  temp = gst_pad_get_allowed_caps (space->srcpad);
+  peercaps = gst_caps_copy (temp);
+  gst_caps_unref (temp);
+
   /* and our own template of course */
-  ourcaps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
+  temp = gst_pad_get_pad_template_caps (pad);
+  ourcaps = gst_caps_copy (temp);
+  gst_caps_unref (temp);
 
   /* merge them together, we prefer the peercaps first */
   result = gst_caps_prepend (ourcaps, peercaps);
@@ -286,7 +291,7 @@ gst_colorspace_getcaps (GstPad *pad, GstCaps *caps)
   return result;
 }
 
-static GstPadConnectReturn
+static GstPadLinkReturn
 gst_colorspace_sinkconnect (GstPad *pad, GstCaps *caps)
 {
   GstColorspace *space;
@@ -295,7 +300,7 @@ gst_colorspace_sinkconnect (GstPad *pad, GstCaps *caps)
   space = GST_COLORSPACE (gst_pad_get_parent (pad));
 
   if (!GST_CAPS_IS_FIXED (caps)) {
-    return GST_PAD_CONNECT_DELAYED;
+    return GST_PAD_LINK_DELAYED;
   }
 
   gst_caps_get_int (caps, "width", &space->width);
@@ -303,31 +308,34 @@ gst_colorspace_sinkconnect (GstPad *pad, GstCaps *caps)
 
   GST_INFO (GST_CAT_PROPERTIES, "size: %dx%d", space->width, space->height);
 
-  space->sinkcaps = caps;
+  gst_caps_replace_sink (&space->sinkcaps, caps);
 
   peer = gst_pad_get_peer (pad);
   if (peer) {
-    if (gst_colorspace_srcconnect_func (pad, gst_pad_get_allowed_caps (space->srcpad), FALSE) < 1) {
+    GstCaps *allowed = gst_pad_get_allowed_caps (space->srcpad);
+    if (gst_colorspace_srcconnect_func (pad, allowed, FALSE) < 1) {
       space->sinkcaps = NULL;
-      return GST_PAD_CONNECT_REFUSED;
+      gst_caps_unref (allowed);
+      return GST_PAD_LINK_REFUSED;
     }
+    gst_caps_unref (allowed);
   }
 
-  return GST_PAD_CONNECT_OK;
+  return GST_PAD_LINK_OK;
 }
 
-static GstPadConnectReturn
+static GstPadLinkReturn
 gst_colorspace_srcconnect (GstPad *pad, GstCaps *caps)
 {
   return gst_colorspace_srcconnect_func (pad, caps, TRUE);
 }
 
-static GstPadConnectReturn
+static GstPadLinkReturn
 gst_colorspace_srcconnect_func (GstPad *pad, GstCaps *caps, gboolean newcaps)
 {
   GstColorspace *space;
   GstCaps *peercaps;
-  GstCaps *ourcaps;
+  GstCaps *ourcaps, *to_intersect, *try_peercaps;
   
   space = GST_COLORSPACE (gst_pad_get_parent (pad));
 
@@ -337,7 +345,7 @@ gst_colorspace_srcconnect_func (GstPad *pad, GstCaps *caps, gboolean newcaps)
     if (newcaps)
       gst_pad_recalc_allowed_caps (space->sinkpad);
 
-    return GST_PAD_CONNECT_DELAYED;
+    return GST_PAD_LINK_DELAYED;
   }
 
   /* first see if we can do the format natively by filtering the peer caps 
@@ -348,20 +356,27 @@ gst_colorspace_srcconnect_func (GstPad *pad, GstCaps *caps, gboolean newcaps)
     if (gst_pad_try_set_caps (space->srcpad, peercaps) > 0) {
       space->type = GST_COLORSPACE_NONE;
       space->disabled = FALSE;
-      return GST_PAD_CONNECT_DONE;
+      gst_caps_unref (peercaps);
+      return GST_PAD_LINK_DONE;
     }
+    gst_caps_unref (peercaps);
   }
-  /* then see what the peer has that matches the size */
-  peercaps = gst_caps_intersect (caps,
-		  GST_CAPS_NEW (
+
+  to_intersect = GST_CAPS_NEW (
 		   "colorspace_filter",
 		   "video/raw",
 		     "width",   GST_PROPS_INT (space->width),
 		     "height",  GST_PROPS_INT (space->height)
-		  ));
+		 );
+
+  /* then see what the peer has that matches the size */
+  peercaps = gst_caps_intersect (caps, to_intersect);
+  gst_caps_unref (to_intersect);
 
   /* we are looping over the caps, so we have to get rid of the lists */
-  peercaps = gst_caps_normalize (peercaps);
+  try_peercaps = gst_caps_normalize (peercaps);
+  gst_caps_unref (peercaps);
+  peercaps = try_peercaps;
 
   /* loop over all possibilities and select the first one we can convert and
    * is accepted by the peer */
@@ -369,16 +384,18 @@ gst_colorspace_srcconnect_func (GstPad *pad, GstCaps *caps, gboolean newcaps)
     if (colorspace_setup_converter (space, ourcaps, peercaps)) {
       if (gst_pad_try_set_caps (space->srcpad, peercaps) > 0) {
         space->disabled = FALSE;
-        return GST_PAD_CONNECT_DONE;
+        gst_caps_unref (try_peercaps);
+        return GST_PAD_LINK_DONE;
       }
     }
     peercaps = peercaps->next;
   }
+  gst_caps_unref (try_peercaps);
   
   /* we disable ourself here */
   space->disabled = TRUE;
 
-  return GST_PAD_CONNECT_REFUSED;
+  return GST_PAD_LINK_REFUSED;
 }
 
 GType
@@ -425,7 +442,7 @@ gst_colorspace_init (GstColorspace *space)
 {
   space->sinkpad = gst_pad_new_from_template (
 		  GST_PAD_TEMPLATE_GET (colorspace_sink_template_factory), "sink");
-  gst_pad_set_connect_function (space->sinkpad, gst_colorspace_sinkconnect);
+  gst_pad_set_link_function (space->sinkpad, gst_colorspace_sinkconnect);
   gst_pad_set_getcaps_function (space->sinkpad, gst_colorspace_getcaps);
   gst_pad_set_bufferpool_function (space->sinkpad, colorspace_get_bufferpool);
   gst_pad_set_chain_function(space->sinkpad,gst_colorspace_chain);
@@ -434,7 +451,7 @@ gst_colorspace_init (GstColorspace *space)
   space->srcpad = gst_pad_new_from_template (
 		  GST_PAD_TEMPLATE_GET (colorspace_src_template_factory), "src");
   gst_element_add_pad(GST_ELEMENT(space),space->srcpad);
-  gst_pad_set_connect_function (space->srcpad, gst_colorspace_srcconnect);
+  gst_pad_set_link_function (space->srcpad, gst_colorspace_srcconnect);
 
 #ifdef HAVE_HERMES
   space->h_handle = Hermes_ConverterInstance (0);
@@ -540,6 +557,8 @@ gst_colorspace_change_state (GstElement *element)
       space->pool = gst_pad_get_bufferpool (space->srcpad);
       break;
     case GST_STATE_PLAYING_TO_PAUSED:
+      if (space->pool)
+	gst_buffer_pool_unref (space->pool);
       space->pool = NULL;
       break;
     case GST_STATE_PAUSED_TO_READY:
@@ -547,7 +566,7 @@ gst_colorspace_change_state (GstElement *element)
       space->converter = NULL;
       space->disabled = TRUE;
       space->type = GST_COLORSPACE_NONE;
-      space->sinkcaps = NULL;
+      gst_caps_replace (&space->sinkcaps, NULL);
       break;
   }
 
