@@ -38,6 +38,7 @@ _gst_xwindow_new (gint width, gint height, gboolean toplevel)
 
   new->disp = XOpenDisplay (NULL);
   if (!new->disp) {
+    g_free (new);
     return NULL;
   }
 
@@ -48,9 +49,9 @@ _gst_xwindow_new (gint width, gint height, gboolean toplevel)
 
   new->root = DefaultRootWindow (new->disp);
 
-  new->win = XCreateWindow (new->disp, DefaultRootWindow (new->disp), 
-		            0, 0, new->width, new->height, 0, CopyFromParent, 
-			    CopyFromParent, CopyFromParent, 0, NULL);
+  new->win = XCreateSimpleWindow (new->disp, DefaultRootWindow (new->disp), 
+		                  0, 0, new->width, new->height, 
+                                  0, 0, new->black);
 
   if (!new->win) {
     XCloseDisplay (new->disp);
